@@ -1,10 +1,13 @@
 from __future__ import division, absolute_import
 
+__xcb_proto_version__ = "1.10"
+
 import functools
 import six
 import struct
+import weakref
 
-from .ffi import ffi, C, bytes_to_cdata, visualtype_to_c_struct
+from .ffi import ffi, C, visualtype_to_c_struct  # noqa
 
 X_PROTOCOL = C.X_PROTOCOL
 X_PROTOCOL_REVISION = C.X_PROTOCOL_REVISION
@@ -28,6 +31,8 @@ XCB_CONN_CLOSED_PARSE_ERR = C.XCB_CONN_CLOSED_PARSE_ERR
 # XCB_CONN_CLOSED_INVALID_SCREEN = C.XCB_CONN_CLOSED_INVALID_SCREEN
 # XCB_CONN_CLOSED_FDPASSING_FAILED = C.XCB_CONN_CLOSED_FDPASSING_FAILED
 
+cffi_explicit_lifetimes = weakref.WeakKeyDictionary()
+
 
 def type_pad(t, i):
     return -i & (3 if t > 4 else t - 1)
@@ -35,7 +40,7 @@ def type_pad(t, i):
 
 class Unpacker(object):
 
-    def __init__(self, cdata, known_max=None, needs_pad=False):
+    def __init__(self, cdata, known_max=None):
         self.cdata = cdata
         self.size = 0
         self.offset = 0
@@ -51,7 +56,8 @@ class Unpacker(object):
             self.buf = ffi.buffer(self.cdata, self.size)
 
     def pad(self, thing):
-        if isinstance(thing, type) and any([issubclass(thing, c) for c in [Struct, Union]]):
+        if isinstance(thing, type) and any(
+                [issubclass(thing, c) for c in [Struct, Union]]):
             if hasattr(thing, "fixed_size"):
                 size = thing.fixed_size
             else:
@@ -74,12 +80,18 @@ class Unpacker(object):
         assert self.offset == 0
         return ffi.cast(typ, self.cdata)
 
+    def copy(self):
+        new = Unpacker(self.cdata, self.known_max)
+        new.offset = self.offset
+        return new
+
 
 def popcount(n):
     return bin(n).count('1')
 
 
 class XcffibException(Exception):
+
     """ Generic XcbException; replaces xcb.Exception. """
     pass
 
@@ -99,11 +111,11 @@ class ConnectionException(XcffibException):
             'accepts.'),
         C.XCB_CONN_CLOSED_PARSE_ERR: (
             'Connection closed, error during parsing display string.'),
-#        C.XCB_CONN_CLOSED_INVALID_SCREEN: (
-#            'Connection closed because the server does not have a screen '
-#            'matching the display.'),
-#        C.XCB_CONN_CLOSED_FDPASSING_FAILED: (
-#            'Connection closed because some FD passing operation failed'),
+        #        C.XCB_CONN_CLOSED_INVALID_SCREEN: (
+        #            'Connection closed because the server does not have a screen '
+        #            'matching the display.'),
+        #        C.XCB_CONN_CLOSED_FDPASSING_FAILED: (
+        #            'Connection closed because some FD passing operation failed'),
     }
 
     def __init__(self, err):
@@ -124,9 +136,12 @@ extensions = {}
 
 # This seems a bit over engineered to me; it seems unlikely there will ever be
 # a core besides xproto, so why not just hardcode that?
+
+
 def _add_core(value, _setup, events, errors):
     if not issubclass(value, Extension):
-        raise XcffibException("Extension type not derived from xcffib.Extension")
+        raise XcffibException(
+            "Extension type not derived from xcffib.Extension")
     if not issubclass(_setup, Struct):
         raise XcffibException("Setup type not derived from xcffib.Struct")
 
@@ -143,14 +158,17 @@ def _add_core(value, _setup, events, errors):
 
 def _add_ext(key, value, events, errors):
     if not issubclass(value, Extension):
-        raise XcffibException("Extension type not derived from xcffib.Extension")
+        raise XcffibException(
+            "Extension type not derived from xcffib.Extension")
     extensions[key] = (value, events, errors)
 
 
 class ExtensionKey(object):
+
     """ This definitely isn't needed, but we keep it around for compatibilty
     with xpyb.
     """
+
     def __init__(self, name):
         self.name = name
 
@@ -163,6 +181,15 @@ class ExtensionKey(object):
     def __ne__(self, o):
         return self.name != o.name
 
+    def to_cffi(self):
+        c_key = ffi.new("struct xcb_extension_t *")
+        c_key.name = name = ffi.new('char[]', self.name.encode())
+        cffi_explicit_lifetimes[c_key] = name
+        # xpyb doesn't ever set global_id, which seems wrong, but whatever.
+        c_key.global_id = 0
+
+        return c_key
+
 
 class Protobj(object):
 
@@ -192,6 +219,7 @@ class Protobj(object):
             setattr(self, k, v)
         return self
 
+
 class Struct(Protobj):
     pass
 
@@ -202,6 +230,7 @@ class Union(Protobj):
 
 class Cookie(object):
     reply_type = None
+
     def __init__(self, conn, sequence, is_checked):
         self.conn = conn
         self.sequence = sequence
@@ -219,17 +248,20 @@ class Cookie(object):
 
 
 class VoidCookie(Cookie):
+
     def reply(self):
         raise XcffibException("No reply for this message type")
 
 
 class Extension(object):
+
     def __init__(self, conn, key=None):
         self.conn = conn
+
         if key is None:
-            self.ext_name = None
+            self.c_key = ffi.NULL
         else:
-            self.ext_name = key.name
+            self.c_key = key.to_cffi()
 
     def send_request(self, opcode, data, cookie=VoidCookie, reply=None,
                      is_checked=False):
@@ -239,24 +271,16 @@ class Extension(object):
 
         xcb_req = ffi.new("xcb_protocol_request_t *")
         xcb_req.count = 2
-
-        if self.ext_name is not None:
-            key = ffi.new("struct xcb_extension_t *")
-            key.name = bytes_to_cdata(self.ext_name.encode())
-            # xpyb doesn't ever set global_id, which seems wrong, but whatever.
-            key.global_id = 0
-            xcb_req.ext = key
-        else:
-            xcb_req.ext = ffi.NULL
-
+        xcb_req.ext = self.c_key
         xcb_req.opcode = opcode
         xcb_req.isvoid = issubclass(cookie, VoidCookie)
 
         xcb_parts = ffi.new("struct iovec[2]")
-        xcb_parts[0].iov_base = bytes_to_cdata(data)
+        xcb_parts[0].iov_base = iov_base = ffi.new('char[]', data)
         xcb_parts[0].iov_len = len(data)
         xcb_parts[1].iov_base = ffi.NULL
         xcb_parts[1].iov_len = -len(data) & 3  # is this really necessary?
+        cffi_explicit_lifetimes[xcb_parts] = iov_base
 
         flags = C.XCB_REQUEST_CHECKED if is_checked else 0
 
@@ -280,6 +304,7 @@ class Extension(object):
 
 
 class List(Protobj):
+
     def __init__(self, unpacker, typ, count=None):
         Protobj.__init__(self, unpacker)
 
@@ -324,7 +349,7 @@ class List(Protobj):
         """ A helper for converting a List of chars to a native string. Dies if
         the list contents are not something that could be reasonably converted
         to a string. """
-        return b''.join(self).decode('latin1')
+        return ''.join(chr(six.byte2int(i)) for i in self)
 
     def to_atoms(self):
         """ A helper for converting a List of chars to an array of atoms """
@@ -333,6 +358,24 @@ class List(Protobj):
     def buf(self):
         return b''.join(self.list)
 
+
+class OffsetMap(object):
+
+    def __init__(self, core):
+        self.offsets = [(0, core)]
+
+    def add(self, offset, things):
+        self.offsets.append((offset, things))
+        self.offsets.sort(key=lambda x: x[0], reverse=True)
+
+    def __getitem__(self, item):
+        try:
+            offset, things = next((k, v) for k, v in self.offsets if item >= k)
+            return things[item - offset]
+        except StopIteration:
+            raise IndexError(item)
+
+
 class Connection(object):
 
     def __init__(self, display=None, fd=-1, auth=None):
@@ -342,10 +385,13 @@ class Connection(object):
                 raise XcffibException("invalid xauth")
         else:
             c_auth = ffi.NULL
-        display = display.encode('latin1')
+
+        if display is None:
+            display = ffi.NULL
+        else:
+            display = display.encode('latin1')
 
         i = ffi.new("int *")
-        i[0] = 0
 
         if fd > 0:
             self._conn = C.xcb_connect_to_fd(fd, c_auth)
@@ -359,6 +405,19 @@ class Connection(object):
         self.core = core(self)
         self.setup = self.get_setup()
 
+        self._event_offsets = OffsetMap(core_events)
+        self._error_offsets = OffsetMap(core_errors)
+        self._setup_extensions()
+
+    def _setup_extensions(self):
+        for key, (_, events, errors) in extensions.items():
+            # We're explicitly not putting this as an argument to the next call
+            # as a hack for lifetime management.
+            c_ext = key.to_cffi()
+            reply = C.xcb_get_extension_data(self._conn, c_ext)
+            self._event_offsets.add(reply.first_event, events)
+            self._error_offsets.add(reply.first_error, errors)
+
     def __call__(self, key):
         return extensions[key][0](self, key)
 
@@ -441,7 +500,7 @@ class Connection(object):
     def _process_error(self, c_error):
         self.invalid()
         if c_error != ffi.NULL:
-            error = core_errors[c_error.error_code]
+            error = self._error_offsets[c_error.error_code]
             buf = Unpacker(c_error)
             raise error(buf)
 
@@ -479,25 +538,26 @@ class Connection(object):
         if e.response_type == 0:
             return self._process_error(ffi.cast("xcb_generic_error_t *", e))
 
-        assert core_events, "You probably need to import xcffib.xproto"
         # We mask off the high bit here because events sent with SendEvent have
         # this bit set. We don't actually care where the event came from, so we
         # just throw this away. Maybe we could expose this, if anyone actually
         # cares about it.
-        event = core_events[e.response_type & 0x7f]
+        event = self._event_offsets[e.response_type & 0x7f]
 
         buf = Unpacker(e)
         return event(buf)
 
+    @ensure_connected
     def send_request(self, flags, xcb_parts, xcb_req):
-        self.invalid()
         return C.xcb_send_request(self._conn, flags, xcb_parts, xcb_req)
 
+
 # More backwards compatibility
 connect = Connection
 
 
 class Response(Protobj):
+
     def __init__(self, unpacker):
         Protobj.__init__(self, unpacker)
 
@@ -510,6 +570,7 @@ class Response(Protobj):
 
 
 class Reply(Response):
+
     def __init__(self, unpacker):
         Response.__init__(self, unpacker)
 
@@ -523,6 +584,7 @@ class Event(Response):
 
 
 class Error(Response, XcffibException):
+
     def __init__(self, unpacker):
         Response.__init__(self, unpacker)
         XcffibException.__init__(self)
@@ -534,33 +596,31 @@ def pack_list(from_, pack_type):
     subclass of `xcffib.Struct`, or a string that can be passed to
     `struct.pack`. You must pass `size` if `pack_type` is a struct.pack string.
     """
-
-    if six.PY3:
-        # If a string is passed as `from_` in Python 3, it has to be encoded
-        if isinstance(from_, str):
-            from_ = [b.encode('latin1') for b in from_]
-        # PY3 is "helpful" in that when you do tuple(b'foo') you get
-        # (102, 111, 111) instead of something more reasonable like
-        # (b'f', b'o', b'o'), so we have to add this other special case.
-        elif isinstance(from_, bytes):
-            from_ = [bytes([b]) for b in from_]
-
-    try:
-        elt_type = type(from_[0])
-    except IndexError:
+    # We need from_ to not be empty
+    if len(from_) == 0:
         return bytes()
 
-    # Pack from_ as char array, where from_ may be an array of ints possibly
-    # greater than 256
-    if pack_type == 'c' and issubclass(elt_type, int):
-        def to_bytes(v):
-            for _ in range(4):
-                v, r = divmod(v, 256)
-                yield r
-        from_ = [bytes(bytearray([b])) for i in from_ for b in to_bytes(i)]
+    if pack_type == 'c':
+        if isinstance(from_, bytes):
+            # PY3 is "helpful" in that when you do tuple(b'foo') you get
+            # (102, 111, 111) instead of something more reasonable like
+            # (b'f', b'o', b'o'), so we rebuild from_ as a tuple of bytes
+            from_ = [six.int2byte(b) for b in six.iterbytes(from_)]
+        elif isinstance(from_, str):
+            # Only run in Python 3, where bytes are different than strings
+            # Here we create the tuple of bytes by encoding each character
+            from_ = [b.encode('latin-1') for b in from_]
+        elif isinstance(from_[0], int):
+            # Pack from_ as char array, where from_ may be an array of ints
+            # possibly greater than 256
+            def to_bytes(v):
+                for _ in range(4):
+                    v, r = divmod(v, 256)
+                    yield r
+            from_ = [six.int2byte(b) for i in from_ for b in to_bytes(i)]
 
     if isinstance(pack_type, six.string_types):
-        return struct.pack("=" + pack_type * len(from_), *tuple(from_))
+        return struct.pack("=" + pack_type * len(from_), *from_)
     else:
         buf = six.BytesIO()
         for item in from_:
